(* $Id: menu_testconf.ml 678 2007-02-25 17:16:54Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

open Godi_types
open Printf


let help_active_str =
  "This function tests whether the package can be configured. If you\n" ^
  "have set any options, these will be used.\n" ^
  "\n" ^
  "Note that this function works only if all required packages are\n" ^
  "already installed (otherwise you will get an error about broken\n" ^
  "dependencies)." ^
  "\n" ^
  "You can abort the configuration trial by pressing 'a'.\n" ;;

let help_over_str =
  "It has been tried to configure the package. On success, a line like\n" ^
  "\n" ^
  "  ### Configuration successful\n" ^
  "\n" ^
  "is displayed; otherwise an error occurred. Note that one common reason\n" ^
  "for errors are missing dependencies: This routine does not install\n" ^
  "required packages first, but simply starts the configuration script.\n" ^
  "Other causes are: You do not have the required libraries on your\n" ^
  "system, or you have forgotten to install the C header files, too\n" ^
  "(many Linux distros have them in separate *-dev rpm/deb archives).\n" ;;


let display (mk_pg : unit -> Miniui.page) pkg () =
  let pg = mk_pg () in
  pg # configure ~auto_scroll:true ();
 
  let poll () =
    pg # update();
    pg # poll_event()
  in

  Menu_aux.add_compact_head ~lmargin:6 pg ">>> Test Configuration <<<";

  pg # put_line
    ("### Testing, whether this package can be configured: " ^
     pkg_name pkg);

  let active = ref true in
              (* set this to false to stop the external process *)
  let disp_proto = new Protocol.display_protocol pg in
  let protocol = disp_proto # collector in
  let ia = { Godi_exec.period = 0.2;
	     Godi_exec.callback = (fun () ->
				     poll();
				     !active);
	   } in

  let cancel_btn =
    pg # add_foot_button
      ~text:"abort"
      ~underline:0
      ~callback:(fun () ->
		   active := false)
      (`Keypress 'a') in

  let help_btn =
    pg # add_foot_button ~text:"help" ~underline:0
      ~callback:(Menu_help.display pg#new_page help_active_str)
      (`Keypress 'h') in

  let tmp_godi_conf = ref "" in

  let ui_continue() =
    if !tmp_godi_conf <> "" then Sys.remove !tmp_godi_conf;
    pg # remove_button cancel_btn;
    pg # remove_button help_btn;
        ignore(pg # add_foot_button
	     ~text:"save protocol"
	     ~underline:0
	     ~callback:(fun () ->
			  disp_proto # save_dialog())
	     (`Keypress 's'));
    ignore(pg # add_foot_button ~text:"help" ~underline:0
	     ~callback:(Menu_help.display pg#new_page help_over_str)
	     (`Keypress 'h'));

    ignore(pg # add_foot_button
	     ~text:"exit menu"
	     ~underline:1
	     ~callback:(fun () -> pg#quit())
	     (`Keypress 'x'));

    pg # loop();
  in

  try
    tmp_godi_conf :=  Filename.temp_file "tmpgodi" ".conf";
    let mflag = Godi_conffile.modify_godi_conf
                  ~pref_user_error:true ~destfile:!tmp_godi_conf [pkg] in
    Godi_perform.test_conf_pkg
      ~pref_user_error:true
      ~interactivity:ia ~protocol ~godi_conf:!tmp_godi_conf pkg;
    pg # put_line "### Configuration successful";
    ui_continue();
  with
      Godi_exec.Command_discontinued ->
	pg # put_line "";
	pg # put_line "### Configuration aborted";
	ui_continue();
    | User_error msg ->
	pg # put_line "";
	pg # put_line ("### Error: " ^ msg);
	ui_continue();
;;
