(* $Id: menu_update.ml 678 2007-02-25 17:16:54Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann. 
 * 
 * This file is part of godi-tools. 
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

open Godi_types
open Printf

let display (pg : Miniui.page) =
  pg # clear();

  pg # configure ~auto_scroll:true ();

  Menu_aux.add_full_head pg ">>> Update Source Packages <<<";

  let poll () =
    pg # update();
    pg # poll_event()
  in

  let active = ref true in
               (* set this to false to stop the update process *)
  let disp_proto = new Protocol.display_protocol ~auto_update:true pg in
  let protocol = disp_proto # collector in
  let ia = { Godi_exec.period = 0.2;
	     Godi_exec.callback = (fun () ->
				     poll();
				     !active);
	   } in

  let cancel_btn =
    pg # add_foot_button 
      ~text:"abort"
      ~underline:0
      ~callback:(fun () ->
		   active := false)
      (`Keypress 'a') in

  pg # set_busy true;
  pg # update();

  ( try
      Godi_update.update_available ~interactivity:ia ~protocol ();
      Godi_update.update_build_instructions ~interactivity:ia ~protocol ();
      pg # put_line "";
      pg # put_line "### Update successful";
    with
	Godi_exec.Command_discontinued ->
	  pg # put_line "";
	  pg # put_line "### Update aborted";
      | User_error msg ->
	  pg # put_line "";
	  pg # put_line ("### Error: " ^ msg)
  );

  pg # update();
  pg # set_busy false;

  Godi_query.forget_source_packages();

  pg # remove_button cancel_btn;
  let exit_btn =
    pg # add_foot_button 
      ~text:"exit"
      ~underline:1
      ~callback:(fun () ->
		   pg#quit())
      (`Keypress 'x') in

  pg # loop();
  ()
;;
